@@ -22,16 +22,16 @@ from difflib import unified_diff
 import tempfile
 import ast
 
-from distutils.version import LooseVersion
+from distutils.version import StrictVersion
 try:
     import pep8
-    if LooseVersion(pep8.__version__) < LooseVersion('0.5.1'):
+    if StrictVersion(pep8.__version__) < StrictVersion('1.3a2'):
         pep8 = None
 except ImportError:
     pep8 = None
 
 
-__version__ = '0.6.5'
+__version__ = '0.7'
 
 
 PEP8_BIN = 'pep8'
@@ -43,7 +43,7 @@ MAX_LINE_WIDTH = 79
 
 
 def open_with_encoding(filename, encoding, mode='r'):
-    """Open file with a specific encoding."""
+    """Return opened file with a specific encoding."""
     try:
         # Python 3
         return open(filename, mode=mode, encoding=encoding)
@@ -71,12 +71,7 @@ def detect_encoding(filename):
 
 
 def read_from_filename(filename, readlines=False):
-    """Simple open file, read contents, close file.
-
-    Ensures file gets closed without relying on CPython GC.
-    Jython requires files to be closed.
-
-    """
+    """Return contents of file."""
     with open_with_encoding(filename,
                             encoding=detect_encoding(filename)) as input_file:
         return input_file.readlines() if readlines else input_file.read()
@@ -132,14 +127,6 @@ class FixPEP8(object):
         self.fix_e274 = self.fix_e271
         self.fix_w191 = self.fix_e101
 
-    def _pep8_options(self, targetfile):
-        """Return options to be passed to pep8."""
-        return (["--repeat", targetfile] +
-                (["--ignore=" + self.options.ignore]
-                 if self.options.ignore else []) +
-                (["--select=" + self.options.select]
-                 if self.options.select else []))
-
     def _fix_source(self, results):
         completed_lines = []
         for result in sorted(results, key=_priority_key):
@@ -168,11 +155,21 @@ class FixPEP8(object):
                                                         info))
 
     def fix(self):
-        pep8_options = self._pep8_options(self.filename)
+        """Return a version of the source code with PEP 8 violations fixed."""
         if pep8:
+            pep8_options = {
+                'ignore':
+                self.options.ignore and self.options.ignore.split(','),
+                'select':
+                self.options.select and self.options.select.split(','),
+            }
             results = _execute_pep8(pep8_options, self.source)
         else:
-            results = _spawn_pep8(pep8_options)
+            results = _spawn_pep8((["--ignore=" + self.options.ignore]
+                                   if self.options.ignore else []) +
+                                  (["--select=" + self.options.select]
+                                   if self.options.select else []) +
+                                  [self.filename])
         self._fix_source(results)
         return "".join(self.source)
 
@@ -216,15 +213,12 @@ class FixPEP8(object):
             return []
 
     def fix_e231(self, result):
-        target = self.source[result['line'] - 1]
-        target_char = result['info'].split()[-1][1]
-        fixed = ""
-        fixed_end = 0
-        for i in re.finditer("%s\S" % target_char, target):
-            fixed += target[fixed_end:i.start()] + "%s " % target_char
-            fixed_end = i.end() - 1
-        fixed += target[fixed_end:]
-        self.source[result['line'] - 1] = fixed
+        """Add missing whitespace."""
+        line_index = result['line'] - 1
+        target = self.source[line_index]
+        offset = result['column']
+        fixed = target[:offset] + ' ' + target[offset:]
+        self.source[line_index] = fixed
 
     def fix_e251(self, result):
         line_index = result['line'] - 1
@@ -611,6 +605,7 @@ class FixPEP8(object):
 
 
 def _find_newline(source):
+    """Return type of newline used in source."""
     cr, lf, crlf = 0, 0, 0
     for s in source:
         if CRLF in s:
@@ -631,6 +626,7 @@ def _find_newline(source):
 
 
 def _get_indentword(source):
+    """Return indentation type."""
     sio = StringIO(source)
     indent_word = "    "  # Default in case source has no indentation
     try:
@@ -644,12 +640,13 @@ def _get_indentword(source):
 
 
 def _get_indentation(line):
+    """Return leading whitespace."""
     non_whitespace_index = len(line) - len(line.lstrip())
     return line[:non_whitespace_index]
 
 
 def _split_indentation(line):
-    """Split into tuple (indentation, rest)."""
+    """Return line split into tuple (indentation, rest)."""
     non_whitespace_index = len(line) - len(line.lstrip())
     return (line[:non_whitespace_index], line[non_whitespace_index:])
 
@@ -805,32 +802,35 @@ def _spawn_pep8(pep8_options):
 
 def _execute_pep8(pep8_options, source):
     """Execute pep8 via python method calls."""
-    pep8.process_options(['pep8'] + pep8_options)
-
-    class QuietChecker(pep8.Checker):
+    class QuietReport(pep8.BaseReport):
 
         """Version of checker that does not print."""
 
-        def __init__(self, filename, lines):
-            pep8.Checker.__init__(self, filename, lines=lines)
-            self.__results = None
+        def __init__(self, options):
+            super(QuietReport, self).__init__(options)
+            self.__full_error_results = []
 
-        def report_error(self, line_number, offset, text, check):
+        def error(self, line_number, offset, text, _):
             """Collect errors."""
-            code = text[:4]
-            if not pep8.ignore_code(code):
-                self.__results.append(
-                    dict(id=text.split()[0], line=line_number,
+            code = super(QuietReport, self).error(line_number, offset, text, _)
+            if code:
+                self.__full_error_results.append(
+                    dict(id=code, line=line_number,
                          column=offset + 1, info=text))
 
-        def check_all(self, expected=None, line_offset=0):
-            """Check code and return results."""
-            self.__results = []
-            pep8.Checker.check_all(self, expected, line_offset)
-            return self.__results
+        def full_error_results(self):
+            """Return error results in detail.
 
-    checker = QuietChecker('', lines=source)
-    return checker.check_all()
+            Results are in the form of a list of dictionaries. Each dictionary
+            contains 'id', 'line', 'column', and 'info'.
+
+            """
+            return self.__full_error_results
+
+    checker = pep8.Checker('', lines=source,
+                           reporter=QuietReport, **pep8_options)
+    checker.check_all()
+    return checker.report.full_error_results()
 
 
 class Reindenter(object):
@@ -888,7 +888,7 @@ class Reindenter(object):
         for i in range(len(stats) - 1):
             thisstmt, thislevel = stats[i]
             nextstmt = stats[i + 1][0]
-            have = _getlspace(lines[thisstmt])
+            have = _leading_space_count(lines[thisstmt])
             want = thislevel * 4
             if want < 0:
                 # A comment line.
@@ -902,7 +902,7 @@ class Reindenter(object):
                         for j in range(i + 1, len(stats) - 1):
                             jline, jlevel = stats[j]
                             if jlevel >= 0:
-                                if have == _getlspace(lines[jline]):
+                                if have == _leading_space_count(lines[jline]):
                                     want = jlevel * 4
                                 break
                     if want < 0:           # Maybe it's a hanging
@@ -912,8 +912,10 @@ class Reindenter(object):
                         for j in range(i - 1, -1, -1):
                             jline, jlevel = stats[j]
                             if jlevel >= 0:
-                                want = (have + _getlspace(after[jline - 1]) -
-                                        _getlspace(lines[jline]))
+                                want = (have +
+                                        _leading_space_count(
+                                            after[jline - 1]) -
+                                        _leading_space_count(lines[jline]))
                                 break
                     if want < 0:
                         # Still no luck -- leave it alone.
@@ -933,7 +935,7 @@ class Reindenter(object):
                         else:
                             after.append(" " * diff + line)
                     else:
-                        remove = min(_getlspace(line), -diff)
+                        remove = min(_leading_space_count(line), -diff)
                         after.append(line[remove:])
         return self.raw != self.after
 
@@ -989,10 +991,10 @@ class Reindenter(object):
                 self.stats.append((sline, self.level))
 
 
-def _getlspace(line):
-    """Count number of leading blanks."""
+def _leading_space_count(line):
+    """Return number of leading spaces in line."""
     i = 0
-    while i < len(line) and line[i] == " ":
+    while i < len(line) and line[i] == ' ':
         i += 1
     return i
 
@@ -1003,13 +1005,15 @@ def fix_file(filename, opts, output=sys.stdout):
     fixed_source = fix.fix()
     original_source = copy.copy(fix.original_source)
     tmp_filename = filename
+    if not pep8 or opts.in_place:
+        encoding = detect_encoding(filename)
     for _ in range(opts.pep8_passes):
         if fixed_source == tmp_source:
             break
         tmp_source = copy.copy(fixed_source)
         if not pep8:
             tmp_filename = tempfile.mkstemp()[1]
-            fp = open(tmp_filename, 'w')
+            fp = open_with_encoding(tmp_filename, encoding=encoding, mode='w')
             fp.write(fixed_source)
             fp.close()
         fix = FixPEP8(tmp_filename, opts, contents=tmp_source)
@@ -1024,7 +1028,7 @@ def fix_file(filename, opts, output=sys.stdout):
         new = new.readlines()
         output.write(_get_difftext(original_source, new, filename))
     elif opts.in_place:
-        fp = open_with_encoding(filename, encoding=detect_encoding(filename),
+        fp = open_with_encoding(filename, encoding=encoding,
                                 mode='w')
         fp.write(fixed_source)
         fp.close()
